package com.keao.alimeeting.detail

import android.content.Context
import android.content.Intent
import android.os.Handler
import android.os.Message
import com.alibaba.fastjson.JSONObject
import com.keao.alimeeting.utils.Platform
import com.aliwork.meeting.api.AMSDKMediaStatusCallBack
import com.aliwork.meeting.api.AMSDKMeetingCallBack
import com.aliwork.meeting.api.AMSDKMeetingConfig
import com.aliwork.meeting.api.AMSDKMeetingManager
import com.aliwork.meeting.api.device.AMSDKDeviceManager
import com.aliwork.meeting.api.device.AMSDKMediaDevice
import com.aliwork.meeting.api.device.AMSDKMediaDeviceChangeCallback
import com.aliwork.meeting.api.member.AMSDKMeetingClient
import com.aliwork.meeting.api.render.AMSDKEglBase

class ConferencePresenter() : RxPresenter<ConferenceView>(), AMSDKMediaDeviceChangeCallback {


    private companion object {
        const val TAG = "ConferencePresenter"
        const val MSG_GET_MEDIA_STATUS = 11111
    }

    private var eglBase: AMSDKEglBase? = null
    private val manager: AMSDKMeetingManager? = AMSDKMeetingManager.create()
    private lateinit var meetingId: String
    private lateinit var userId:String
    private lateinit var memberUuid: String
    private lateinit var clientAppId: String
    private var debugInfoHandler: DebugInfoHandler? = null
    private var isInBlackVideoMode =false
    private var closeCamera = false
    private var muteAudio = false
    private var disableVideo = false
    private var cameraOpen = true

    fun initMeetingInfo(intent : Intent, userId: String, meetingId: String,
                        memberUuid: String, clientAppId: String) {
        this.userId = userId
        this.meetingId = meetingId
        this.memberUuid = memberUuid
        this.clientAppId = clientAppId
        closeCamera = intent.getBooleanExtra("closeCamera",false)
        muteAudio = intent.getBooleanExtra("muteAudio", false)
        disableVideo = intent.getBooleanExtra("disableVideo", false)
    }

    fun start(context: Context, meetingToken: String, meetingDomain: String, eglBase: AMSDKEglBase,
        callback: AMSDKMeetingCallBack
    ) {
        //一定要记得Remove，不然可能造成内存泄漏
        AMSDKDeviceManager.getManager(context).addMediaDeviceChangeCallBack(this)

        this.eglBase = eglBase
        val config =
            AMSDKMeetingConfig.Builder(context, meetingId)
                .setMemberUuid(memberUuid)
                .setEglBase(eglBase)
                .setAppId(clientAppId)
                .setUserId(userId)
                .setMeetingToken(meetingToken)
                .setMeetingDomain(meetingDomain)
                .setDefaultMuteAudio(muteAudio)
                .setOpenCameraDefault(!closeCamera)
                .setOpenSpeakerDefault(false)
                .setSecurityTransport(false)
                .setSupportVideo(!disableVideo)
                .build()
        manager?.joinMeeting(config, callback)

        cameraOpen = !closeCamera
    }

    fun triggerAMute() {
        val client = manager?.getPublisherClient() ?: return
        client.muteAudio(client.isAudioOn(), null)
    }

    fun triggerSpeakOn() {
        val deviceManager = AMSDKDeviceManager.getManager(Platform.appContext)
        deviceManager.enableSpeaker(!deviceManager.isSpeakerEnabled())
    }

    fun triggerBlackVideo(black: Boolean) {
        //开启省流量模式，注意要自己保存一个标志位，在省流量模式下，后入会的成员依旧是默认开启视频的，需要自己手动关闭
        if (black) {
            isInBlackVideoMode = true
            manager?.getOnlineClients()
                ?.forEach { it.enableVideo(false, null) }
        } else {
            isInBlackVideoMode = false
            manager?.getOnlineClients()
                ?.forEach { it.enableVideo(true, null) }
        }
    }

    fun isInBlackVideoMode() = isInBlackVideoMode

    fun triggerOpenCamera() {
        cameraOpen = !cameraOpen
        AMSDKDeviceManager.getManager(Platform.appContext).enableCamera(cameraOpen)
    }

    //调用leaveMeeting 或者 finishMeeting，不用再关注 renderView 是否释放，内部会统一释放掉
    fun triggerHangUp() {
        manager?.leaveMeeting()
    }

    fun triggerHangUpAll() {
        manager?.finishMeeting()
    }

    fun triggerSwitchCamera() {
        AMSDKDeviceManager.getManager(Platform.appContext).switchCamera()
    }

    //一场会议里面暂时只能开启一个高清流，调用时不用再关闭上一个高清流，内部会自动关掉上一个高清流
    fun enableHDVideo(client: AMSDKMeetingClient, enable: Boolean) {
        manager?.enableHDVideo(client, enable)
    }

    private fun getMeetingManger(): AMSDKMeetingManager? {
        return manager
    }

    fun startMediaStatus(){
        if (debugInfoHandler == null) {
            debugInfoHandler = DebugInfoHandler()
        }
        debugInfoHandler?.sendEmptyMessage(MSG_GET_MEDIA_STATUS)
    }

    override fun onMediaDeviceChange(isAdd: Boolean, device: AMSDKMediaDevice) {
        //音视频设备发生变化，暂时无效
    }

    override fun onVideoInputDeviceChanged(device: AMSDKMediaDevice) {
        //视频采集设备发生变化，如切换摄像头，通过 device.facing 可以判断前后置摄像头，外置摄像头facing为UNSPECIFIED
    }

    override fun onAudioOutputDeviceChanged(device: AMSDKMediaDevice) {
        //音频输出设备发生变化，device 类型为 AMSDKAudioOutputDevice
        //如果要准确定位设备，首先通过device.transportType 判断 是否为系统内置(TRANSPORT_TYPE_BUILT_IN)
        //   或者TRANSPORT_TYPE_USB、蓝牙(TRANSPORT_TYPE_BLUETOOTH) 其他类型暂不支持
        //如果是系统内置，还可以通过outputPort 进一步判断是否听筒或者扬声器

        //此处可以更改界面扬声器状态，快捷方法通过 AMSDKDeviceManager#isSpeakerEnabled 进行判断
    }

    override fun onDestroyed(){
        super.onDestroyed()

        AMSDKDeviceManager.getManager(Platform.appContext).removeMediaDeviceChangeCallBack(this)
        if (debugInfoHandler != null) {
            debugInfoHandler?.enableLoop = false
            debugInfoHandler?.removeCallbacksAndMessages(null)
            debugInfoHandler = null
        }
    }


    private inner class DebugInfoHandler : Handler(), AMSDKMediaStatusCallBack {

        internal var enableLoop = true

        override fun handleMessage(msg: Message) {
            super.handleMessage(msg)
            if (msg.what == MSG_GET_MEDIA_STATUS) {
                val manager = getMeetingManger()
                if (manager != null && enableLoop) {
                    manager.getMediaStats(this)
                }
            }
        }

        private fun handleResult(result: JSONObject?) {
            val view = getView()
            if (view != null) {
                view.postRunnable(Runnable{
                    try {
                        view.updateMediaStatus(result)
                    } catch (e: Exception) {
                        e.printStackTrace()
                    }
                })
            }
        }

        override fun onCompleted(success: Boolean, result: JSONObject?) {
            handleResult(result)
            if (enableLoop) {
                this.sendEmptyMessageDelayed(MSG_GET_MEDIA_STATUS, 2000)
            }
        }
    }
}


interface ConferenceView {
    fun onMeetingFinished()
    fun updateMediaStatus(result: JSONObject?)

    fun postRunnable(runnable: Runnable)
}