package com.keao.alimeeting.widget;

import android.content.Context;
import android.graphics.PointF;
import android.hardware.SensorManager;
import android.support.v7.widget.GridLayoutManager;
import android.support.v7.widget.LinearSmoothScroller;
import android.support.v7.widget.RecyclerView;
import android.util.AttributeSet;
import android.view.View;
import android.view.ViewConfiguration;

public class PageGridLayoutManager extends GridLayoutManager implements PageLayoutManager {

    // These variables are from android.widget.Scroller, which is used, via ScrollerCompat, by
    // Recycler View. The scrolling distance calculation logic originates from the same place. Want
    // to use their variables so as to approximate the look of normal Android scrolling.
    // Find the Scroller fling implementation in android.widget.Scroller.fling().
    private static final float INFLEXION = 0.35f; // Tension lines cross at (INFLEXION, 1)
    private static float DECELERATION_RATE = (float) (Math.log(0.78) / Math.log(0.9));
    private static double FRICTION = 0.84;
    private double deceleration;
    private int pageSize;
    private int spanRow;
    private OnPageChangeListener onPageChangeListener;
    private int curPage = 0;
    private int totalPage = 0;


    public PageGridLayoutManager(Context context, AttributeSet attrs, int defStyleAttr,
                                 int defStyleRes) {
        super(context, attrs, defStyleAttr, defStyleRes);
        calculateDeceleration(context);
    }

    /**
     * Creates a vertical GridLayoutManager
     *
     * @param context   Current context, will be used to access resources.
     * @param spanCount The number of columns in the grid
     */
    public PageGridLayoutManager(Context context, int spanCount) {
        super(context, spanCount);
        calculateDeceleration(context);
    }

    /**
     * @param context       Current context, will be used to access resources.
     * @param spanCount     The number of columns or rows in the grid
     * @param orientation   Layout orientation. Should be {@link #HORIZONTAL} or {@link
     *                      #VERTICAL}.
     * @param reverseLayout When set to true, layouts from end to start.
     */
    public PageGridLayoutManager(Context context, int spanCount, int orientation,
                                 boolean reverseLayout) {
        super(context, spanCount, orientation, reverseLayout);
        calculateDeceleration(context);
    }


    public void setPageSize(int spanRow, int spanColumn) {
        setSpanCount(spanColumn);
        this.spanRow = spanRow;
        pageSize = spanRow * spanColumn;
        update();
    }


    private void calculateDeceleration(Context context) {
        deceleration = SensorManager.GRAVITY_EARTH // g (m/s^2)
                * 39.3700787 // inches per meter
                // pixels per inch. 160 is the "default" dpi, i.e. one dip is one pixel on a 160 dpi
                // screen
                * context.getResources().getDisplayMetrics().density * 160.0f * FRICTION;
    }

    @Override
    public int getPositionForVelocity(int velocityX, int velocityY) {
        if (getChildCount() == 0) {
            return 0;
        }
        if (getOrientation() == HORIZONTAL) {
            return computeScrollPosition(
                    calcPosForVelocity(velocityX, getChildAt(0).getLeft(), getChildAt(0).getWidth(),
                            getPosition(getChildAt(0))));
        } else {
            return computeScrollPosition(
                    calcPosForVelocity(velocityY, getChildAt(0).getTop(), getChildAt(0).getHeight(),
                            getPosition(getChildAt(0))));
        }
    }

    private int calcPosForVelocity(int velocity, int scrollPos, int childSize, int currPos) {
        final double dist = getSplineFlingDistance(velocity);
        double actualScroll = scrollPos + (int) (velocity > 0 ? dist : -dist);

        //
        int page = (int) (actualScroll / childSize) + (
                spanRow % 2 != 0 && Math.abs(scrollPos) < childSize / 2 ? -1 : 0);

        if (velocity < 0) {
            return Math.max(currPos + page * getSpanCount(), 0);
        } else {
            return Math.min(currPos + page * getSpanCount(), getItemCount());
        }
    }

    @Override
    public void smoothScrollToPosition(RecyclerView recyclerView, RecyclerView.State state,
                                       int position) {
        final LinearSmoothScroller linearSmoothScroller =
                new LinearSmoothScroller(recyclerView.getContext()) {

                    // I want a behavior where the scrolling always snaps to the beginning of
                    // the list. Snapping to end is also trivial given the default implementation.
                    // If you need a different behavior, you may need to override more
                    // of the LinearSmoothScrolling methods.
                    protected int getHorizontalSnapPreference() {
                        return SNAP_TO_START;
                    }

                    protected int getVerticalSnapPreference() {
                        return SNAP_TO_START;
                    }

                    @Override
                    public PointF computeScrollVectorForPosition(int targetPosition) {
                        return PageGridLayoutManager.this
                                .computeScrollVectorForPosition(targetPosition);
                    }
                };
        linearSmoothScroller.setTargetPosition(position);
        startSmoothScroll(linearSmoothScroller);
    }


    int computeScrollPosition(int targetPosition) {
        int page = targetPosition / pageSize;
        //计算目标位置在该页面的行数，如果超过半页则移到下一页
        int row = (targetPosition + 1) % pageSize / getSpanCount();
        if (row > spanRow / 2 - 1 && page < totalPage - 1) {
            page += 1;
        }

        if (page < curPage - 1) {
            page = curPage - 1;
        } else if (page > curPage + 1) {
            page = curPage + 1;
        }

        if (curPage != page) {
            curPage = page;
            if (onPageChangeListener != null) {
                onPageChangeListener.onPageSelected(page);
            }
        }
        return page * pageSize;
    }


    private double getSplineFlingDistance(double velocity) {
        final double l = getSplineDeceleration(velocity);
        final double decelMinusOne = DECELERATION_RATE - 1.0;
        return ViewConfiguration.getScrollFriction() * deceleration
                * Math.exp(DECELERATION_RATE / decelMinusOne * l);
    }

    private double getSplineDeceleration(double velocity) {
        return Math.log(INFLEXION * Math.abs(velocity)
                / (ViewConfiguration.getScrollFriction() * deceleration));
    }

    @Override
    public int getFixScrollPos() {
        if (this.getChildCount() == 0) {
            return 0;
        }

        final View child = getChildAt(0);
        final int childPos = getPosition(child);

        //基数行时，要计算滑到中间时是否滑到下一行
        if (spanRow % 2 != 0) {
            if (getOrientation() == HORIZONTAL
                    && Math.abs(child.getLeft()) > child.getMeasuredWidth() / 2) {
                // Scrolled first view more than halfway offscreen
                return childPos + getSpanCount();
            } else if (getOrientation() == VERTICAL
                    && Math.abs(child.getTop()) > child.getHeight() / 2) {
                // Scrolled first view more than halfway offscreen
                return childPos + getSpanCount();
            }
        }
        return computeScrollPosition(childPos);
    }

    @Override
    public void setOnPageChangeListener(OnPageChangeListener listener) {
        onPageChangeListener = listener;
    }

    @Override
    public int getCurrentPage() {
        return curPage;
    }

    @Override
    public int getTotalPage() {
        return totalPage;
    }

    public void update() {
        // 计算总页数
        int pageCount = (int) Math.ceil(getItemCount() / (double) pageSize);
        if (pageCount != totalPage) {
            if (onPageChangeListener != null) {
                onPageChangeListener.onPageCountChanged(pageCount);
            }
            // 页码减少且当前页为最后一页
            if (pageCount < totalPage && curPage >= pageCount) {
                curPage = pageCount - 1;
                // 执行滚动
//                smoothScrollBy(0, -getHeight());
            }
            if (onPageChangeListener != null) {
                onPageChangeListener.onPageSelected(curPage);
            }
            totalPage = pageCount;
        }
    }


}